<?php

namespace Uncanny_Automator;

/**
 * Class LP_MARKSECTIONDONE
 *
 * @package uncanny_automator
 */
class LP_MARKSECTIONDONE {

	/**
	 * Integration code
	 *
	 * @var string
	 */
	public static $integration = 'LP';

	private $action_code;
	private $action_meta;

	/**
	 * Set up Automator action constructor.
	 */
	public function __construct() {
		$this->action_code = 'LPMARKSECTIONDONE-A';
		$this->action_meta = 'LPSECTION';
		$this->define_action();
	}

	/**
	 * Define and register the action by pushing it into the Automator object
	 */
	public function define_action() {

		global $uncanny_automator;
		$args    = [
			'post_type'      => 'lp_course',
			'posts_per_page' => 999,
			'orderby'        => 'title',
			'order'          => 'ASC',
			'post_status'    => 'publish',
		];
		$options = $uncanny_automator->helpers->recipe->options->wp_query( $args, false, 'course' );

		$action = [
			'author'             => $uncanny_automator->get_author_name( $this->action_code ),
			'support_link'       => $uncanny_automator->get_author_support_link( $this->action_code ),
			'integration'        => self::$integration,
			'code'               => $this->action_code,
			/* Translators: 1:Courses*/
			'sentence'           => sprintf( __( 'Mark {{a section:%1$s}} complete', 'uncanny-automator' ), $this->action_meta ),
			'select_option_name' => __( 'Mark {{a section}} complete', 'uncanny-automator' ),
			'priority'           => 10,
			'accepted_args'      => 1,
			'execution_function' => [ $this, 'lp_mark_section_done' ],
			'options_group'      => [
				$this->action_meta => [
					$uncanny_automator->helpers->recipe->field->select_field_ajax(
						'LPCOURSE',
						__( 'Select a Course', 'uncanny-automator' ),
						$options,
						'',
						'',
						false,
						true,
						[
							'target_field' => 'LPSECTION',
							'endpoint'     => 'select_section_from_course_LPMARKLESSONDONE',
						]
					),
					$uncanny_automator->helpers->recipe->field->select_field( $this->action_meta, __( 'Select a Section', 'uncanny-automator' ), [ '' => __( 'Select a Course Above', 'uncanny-automator' ) ] ),
				],
			],
		];

		$uncanny_automator->register->action( $action );
	}


	/**
	 * Validation function when the action is hit.
	 *
	 * @param string $user_id user id.
	 * @param array $action_data action data.
	 * @param string $recipe_id recipe id.
	 */
	public function lp_mark_section_done( $user_id, $action_data, $recipe_id ) {

		global $uncanny_automator;

		if ( ! function_exists( 'learn_press_get_current_user' ) ) {
			$error_message = 'The function learn_press_get_current_user does not exist';
			$uncanny_automator->complete_action( $user_id, $action_data, $recipe_id, $error_message );

			return;
		}
		$user = learn_press_get_user( $user_id );

		$course_id  = $action_data['meta']['LPCOURSE'];
		$section_id = $action_data['meta'][ $this->action_meta ];
		// Get All lessons from section.
		$course_curd = new \LP_Section_CURD( $course_id );
		$lessons     = $course_curd->get_section_items( $section_id );
		// Mark lesson completed.
		foreach ( $lessons as $lesson ) {
			if ( $lesson['type'] === 'lp_lesson' ) {
				$result = $user->complete_lesson( $lesson['id'], $course_id );
			} elseif ( $lesson['type'] === 'lp_quiz' ) {
				$quiz_id = $lesson['id'];
				$user    = \LP_Global::user();

				if ( ! $user->has_item_status( [ 'started', 'completed' ], $quiz_id, $course_id ) ) {
					$quiz_data = $user->start_quiz( $quiz_id, $course_id, false );
					$item      = new \LP_User_Item_Course( $quiz_data );
					$item->finish();
				} else {
					$quiz_data = $user->get_item_data( $quiz_id, $course_id );
					$quiz_data->finish();
				}
			}
		}

		if ( ! is_wp_error( $result ) ) {
			$uncanny_automator->complete_action( $user_id, $action_data, $recipe_id );
		} else {
			$error_message = $result->get_error_message();
			$uncanny_automator->complete_action( $user_id, $action_data, $recipe_id, $error_message );

			return;
		}

	}

}
